@@ -4,6 +4,7 @@ from botocore.exceptions import ClientError
 from contextlib import contextmanager
 import subprocess
 import itertools
+import datadiff
 import requests
 import tempfile
 import zipfile
@@ -91,6 +92,16 @@ class LambdaMaker(object):
             else:
                 raise
 
+        old_by_sid = {p["Sid"]: p for p in old_policy}
+        for p in new_policy:
+            sid = p["Sid"]
+            if sid in old_by_sid and old_by_sid[sid] != p:
+                log.warning(lc("A policy changes details but keeps the same sid. This means the change will be ignored"
+                    , arn = existing['FunctionArn']
+                    , sid = sid
+                    )
+                )
+
         code_difference = ""
         with into.code_options() as code:
             location = client.get_function(FunctionName=into.name)["Code"]["Location"]
@@ -140,8 +151,10 @@ class LambdaMaker(object):
                     client.add_permission(**trigger.permissions(arn))
 
     def apply_permissions(self, client, arn, into, old_policy):
+        old_sids = [p["Sid"] for p in old_policy]
         for t in into.triggers:
-            client.add_permission(**t.permissions(arn))
+            if t.sid not in old_sids:
+                client.add_permission(**t.permissions(arn))
 
         new_sids = [t.sid for t in into.triggers]
         for s in old_policy:
@@ -153,7 +166,11 @@ class LambdaMaker(object):
             if key not in frm:
                 print("+ {0} = {1}".format(key, printed(val)))
             elif frm[key] != val:
-                print("M {0}\n\tWAS = {1}\n\tINTO = {2}".format(key, printed(frm[key]), printed(val)))
+                if type(frm[key]) is str or type(val) is str:
+                    print("M {0}\n\t- {1}\n\t+ {2}".format(key, frm[key], val))
+                else:
+                    diff = str(datadiff.diff(frm[key], val, fromfile="Existing", tofile="Updated"))
+                    print("M {0}\n\t{1}".format(key, "\n\t".join(line for line in diff.split("\n"))))
 
         for key, val in frm.items():
             if key not in into:
